from .shared.call import call
from .shared.packages import list_packages
from .shared.path import change_dir
from .shared.path import path_factory

import argparse
import sys


def main():
    parser = argparse.ArgumentParser(
        description="Call a script on all repositories listed in a packages.txt.",
        epilog="Additional optional arguments are passed directly to the script.",
    )
    parser.add_argument(
        "script",
        type=path_factory("script", has_extension=".py"),
        help="path to the Python script to be called",
    )
    parser.add_argument(
        "packages_txt",
        type=path_factory("packages.txt", has_extension=".txt"),
        help="path to packages.txt; script is called on each repository listed"
        " inside",
        metavar="packages.txt",
    )
    parser.add_argument(
        "clones",
        type=path_factory("clones", is_dir=True),
        help="path to the directory where the clones of the repositories are stored",
    )

    # idea from https://stackoverflow.com/a/37367814/8531312
    args, sub_args = parser.parse_known_args()
    packages = list_packages(args.packages_txt)

    for package in packages:
        print(f"*** Running {args.script.name} on {package} ***")
        if (args.clones / package).exists():
            with change_dir(args.clones / package):
                print("Updating existing checkout …")
                call("git", "stash")
                call("git", "checkout", "master")
                call("git", "pull")
        else:
            with change_dir(args.clones):
                print("Cloning repository …")
                call("git", "clone", f"https://github.com/zopefoundation/{package}")

        call_args = [sys.executable, args.script, args.clones / package]
        call_args.extend(sub_args)
        call(*call_args)



from copy import deepcopy


def cleanup_data_for_jinja(data):
    """Take a dictionary and cleanup before passing to Jinja.

    This makes it easier to insert in a Jinja template.
    Note: Initially this changed the dictionary inline, but then any changes
    are also written back to .meta.toml, if that is where the dictionary comes from.
    So return a new dictionary.

    * Fixes multi line strings.

      With this as input:

        classifiers = '''
            Environment :: Web Environment
            Framework :: Plone
        '''

      toml reads it as:

        '    Environment :: Web Environment\n    Framework :: Plone\n'

      We want to have the initial newline back.

    """
    if not data:
        return
    new_data = deepcopy(data)
    for key, value in new_data.items():
        if not isinstance(value, str):
            continue
        if "\n" in value and value.startswith(" "):
            new_data[key] = "\n" + value
    return new_data



import pathlib


def list_packages(path: pathlib.Path) -> list:
    """List the packages in ``path``.

    ``path`` must point to a packages.txt file.
    """
    return [p for p in path.read_text().split("\n") if p and not p.startswith("#")]
